
#ifndef _WINSVC_H_
#define _WINSVC_H_
// ----------------------------------------------------------------------------

#include "windows.h"
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>

// ----------------------------------------------------------------------------
// Typedefs to be able to use STL string class in stead of windows LPCTSTR
// types etc, without losing flexibility offered by TCHAR macros
#ifdef UNICODE
typedef std::wstring tstring;
typedef std::wistringstream tistringstream;
typedef std::wostringstream tostringstream;
typedef std::wofstream tofstream;
typedef std::wostream tostream;
#else
typedef std::string tstring;
typedef std::istringstream tistringstream;
typedef std::ostringstream tostringstream;
typedef std::ofstream tofstream;
typedef std::ostream tostream;
#endif
// ----------------------------------------------------------------------------

typedef enum {
	WINSVC_CONSOLE,
	WINSVC_SERVICE,
} etWinSvcMode;

// ----------------------------------------------------------------------------
// Simple class to pass Windows API exception
class ExWinAPI
{
public:
	ExWinAPI() { m_ErrCode = ::GetLastError(); }

	DWORD GetErrCode() { return m_ErrCode; }
	tstring GetErrMsg();

private:
	DWORD m_ErrCode;
};

// ----------------------------------------------------------------------------
// Class WinSvc definition

class WinSvc
{
public:

	// Construction and destruction
	WinSvc(	const tstring& tsServiceName, 
			const tstring& tsEventLogDLL = TEXT(""),
			const tstring& tsLogPath = TEXT(""));
	virtual ~WinSvc();

	// Overload these functions, to let your service do something sensible
	virtual void OnStop() {};
	virtual void OnPause() {};
	virtual void OnContinue() {};
	virtual void OnInterrogate() {};
	virtual void OnShutdown() {};
	virtual void OnParamChange() {};
	virtual void OnNetBindAdd() {};
	virtual void OnNetBindRemove() {};
	virtual void OnNetBindEnable() {};
	virtual void OnNetBindDisable() {};
	virtual void OnDeviceEvent() {};
	virtual void OnHardwareProfileChange() {};
	virtual void OnPowerEvent() {};
	virtual void OnSessionChange() {};

	virtual void OnRun() {};
	virtual void OnExit() {};

	// Overload this function if you need different startup behavior
	virtual bool Start(etWinSvcMode m_etSvcMode = WINSVC_SERVICE);
	
	// Create WIndows Service (takes also care of registration in registry)
	bool Create(const tstring& tsDescription, const tstring& tsFullExePath,
				   int nTimeOut, DWORD dwResetPeriod = INFINITE);
	
	// Remove Windows Service (takes also care of removing from registry)
	bool Remove();

	// Logging
	void OpenLogFile(const tstring& tsLogPath);

private:

	// Entry point for Windows API routines
	static void WINAPI ServiceMain (DWORD dwArgc, LPTSTR* lpszArgv);
	static void WINAPI SvcControlHandler(DWORD dwControl);
	static int  WINAPI ExitRoutine(unsigned long ulControlType);	

	// Entry point for SCM
	void SvcMain();

	// Helper functions to get handles and status
	bool IsSvcRunning();
	const SC_HANDLE& GetSCManagerHandle();
	const SC_HANDLE& GetSvcHandle();

	// Start SCM Dispatcher
	void SvcStartDispatcher();

	// Install special exit handler
	bool InstallExitHandler();
public:
	// Add event source to registry, so service can write to Windows EventLogger
	void WinSvc::AddEventSourceToRegistry(
					const tstring& tsLogName,	const tstring& tsSourceName,
					const tstring& tsFullDllName, DWORD dwCategoryNum);

	// Get reference to internal logger
	tostream& GetLogger();

protected:
	// Report current status to SCM
	void SvcReportStatus(DWORD dwState, DWORD dwExitCode, DWORD dwHint);

	// Service configuration (description and failure actions)
	void SetDescription(const tstring& tsDecription);
	void SetFailActions(DWORD dwResetPeriod, int nTimeOut);

	void LogEvent(const tstring& tsEventSourceName,	DWORD dwEventID, 
		WORD wEventLogType, WORD  wNumParams,	
		const tstring& tsParamData);


private:
	//------------------
	// Member variables
	//------------------

	// Pointer is used to access class from within Windows API functions
	static WinSvc* m_pThis;	

	// Internal service status (not used for console)
	bool m_bSvcRunning;
	SERVICE_STATUS m_SvcStatus;

	// Service identification
	tstring m_tsName;				// Name of service
	tstring m_tsDescription;		// Service description as displayed by SCM

	// Dispatchtable
	SERVICE_TABLE_ENTRY* m_pDispatchTable;

	// Handles
	HANDLE m_hSvcStopEvent;
	HANDLE m_hExitEvent;
	SC_HANDLE m_hSvc;
	SC_HANDLE m_hSCM;

	SERVICE_STATUS_HANDLE m_hSvcStatus;

protected:

	// Identify whether running from console or as service
	etWinSvcMode m_etSvcMode;
	tofstream* m_pLogStream; 
};

// ----------------------------------------------------------------------------
#endif // !_WINSVC_H_
// ----------------------------------------------------------------------------